aboutsummaryrefslogtreecommitdiff
path: root/arch/x86_64/include
diff options
context:
space:
mode:
Diffstat (limited to 'arch/x86_64/include')
-rw-r--r--arch/x86_64/include/arch/memory/allocator/area_frame_allocator.hpp28
-rw-r--r--arch/x86_64/include/arch/memory/multiboot/memory_map.hpp146
-rw-r--r--arch/x86_64/include/arch/memory/paging/page_mapper.hpp2
3 files changed, 144 insertions, 32 deletions
diff --git a/arch/x86_64/include/arch/memory/allocator/area_frame_allocator.hpp b/arch/x86_64/include/arch/memory/allocator/area_frame_allocator.hpp
index a1b771e..8e971f0 100644
--- a/arch/x86_64/include/arch/memory/allocator/area_frame_allocator.hpp
+++ b/arch/x86_64/include/arch/memory/allocator/area_frame_allocator.hpp
@@ -45,22 +45,6 @@ namespace teachos::arch::memory::allocator
*/
auto deallocate_frame(physical_frame physical_frame) -> void;
- /**
- * @brief Returns the iterator pointing to the first element of the memory area.
- * Allows using this class in the for each loop, because it follows the InputIterator template scheme.
- *
- * @return Iterator pointing to first element of the memory area.
- */
- auto begin() -> multiboot::memory_area_iterator;
-
- /**
- * @brief Returns the iterator pointing to one past the last element of the memory area.
- * Allows using this class in the for each loop, because it follows the InputIterator template scheme.
- *
- * @return Iterator pointing to one past the last element of the memory area.
- */
- auto end() -> multiboot::memory_area_iterator;
-
private:
/**
* @brief Find the next memory area and write it into current_area.
@@ -69,12 +53,12 @@ namespace teachos::arch::memory::allocator
physical_frame next_free_frame; ///< The physical_frame after the last allocated one.
std::optional<multiboot::memory_area> current_area; ///< The current memory area.
- multiboot::memory_area_iterator area_begin; ///< Pointer to the first element of all memory areas.
- multiboot::memory_area_iterator area_end; ///< Pointer to one pas the last element of all memory areas.
- physical_frame const kernel_start; ///< The start address of the kernel code in memory.
- physical_frame const kernel_end; ///< The end address of the kernel code in memory.
- physical_frame const multiboot_start; ///< The start address of the multiboot code in memory.
- physical_frame const multiboot_end; ///< The end address of the multiboot code in memory.
+ multiboot::memory_area_container const
+ memory_areas; ///< All memory areas in custom container allows to use std::ranges
+ physical_frame const kernel_start; ///< The start address of the kernel code in memory.
+ physical_frame const kernel_end; ///< The end address of the kernel code in memory.
+ physical_frame const multiboot_start; ///< The start address of the multiboot code in memory.
+ physical_frame const multiboot_end; ///< The end address of the multiboot code in memory.
};
} // namespace teachos::arch::memory::allocator
diff --git a/arch/x86_64/include/arch/memory/paging/page_mapper.hpp b/arch/x86_64/include/arch/memory/paging/page_mapper.hpp
index 4905e9e..7a8e2c9 100644
--- a/arch/x86_64/include/arch/memory/paging/page_mapper.hpp
+++ b/arch/x86_64/include/arch/memory/paging/page_mapper.hpp
@@ -95,7 +95,7 @@ namespace teachos::arch::memory::paging
auto map_next_free_page_to_frame(T & allocator, virtual_page page, std::bitset<64U> flags) -> void
{
auto frame = allocator.allocate_frame();
- exception_handling::assert(!frame.has_value(), "[Page mapper] Out of memory exception");
+ exception_handling::assert(frame.has_value(), "[Page mapper] Out of memory exception");
map_page_to_frame(allocator, page, frame.value(), flags);
}
diff --git a/arch/x86_64/include/arch/memory/multiboot/memory_map.hpp b/arch/x86_64/include/arch/memory/multiboot/memory_map.hpp
index e30d2c4..f9c902a 100644
--- a/arch/x86_64/include/arch/memory/multiboot/memory_map.hpp
+++ b/arch/x86_64/include/arch/memory/multiboot/memory_map.hpp
@@ -3,6 +3,7 @@
#include "info.hpp"
#include <cstdint>
+#include <iterator>
namespace teachos::arch::memory::multiboot
{
@@ -44,48 +45,175 @@ namespace teachos::arch::memory::multiboot
};
/**
* @brief Iterator for memory areas.
*/
struct memory_area_iterator
{
+ using iterator_category = std::random_access_iterator_tag; ///< Iterator category of this type.
+ using difference_type = std::ptrdiff_t; ///< Type when diving one instance of this iterator by another.
+ using value_type = memory_area; ///< Underlying value pointed to by this iterator.
+
+ /**
+ * @brief Defaulted constructor.
+ */
+ memory_area_iterator() = default;
+
/**
* @brief Constructor.
*
- * @param p Underlying address the iterator should point too, ensure to not pass an invalid pointer.
+ * @param p Underlying address the iterator should point too, ensure to not pass an invalid pointer or the
+ * constructor will halt execution.
*/
- explicit memory_area_iterator(memory_area * p);
+ explicit memory_area_iterator(value_type * p);
/**
* @brief Dereferences the initally given pointer to its value.
*
* @return Reference to the value.
*/
- memory_area & operator*() const;
+ auto operator*() const -> value_type &;
+
+ /**
+ * @brief Get underlying value, which is the intially passed pointer.
+ *
+ * @return Underlying value passed intially.
+ */
+ auto operator->() const -> value_type *;
/**
* @brief Post increment operator. Returns a copy of the value.
*
* @return Copy of the incremented underlying address.
*/
- memory_area_iterator operator++(int);
+ auto operator++(int) -> memory_area_iterator;
/**
* @brief Pre increment operator. Returns a reference to the changed value.
*
* @return Reference to the incremented underlying address.
*/
- memory_area_iterator & operator++();
+ auto operator++() -> memory_area_iterator &;
+
+ /**
+ * @brief Addition assignment operator. Returns a reference to the changed value.
+ *
+ * @param value Value we want to add to the underlying address.
+ * @return Reference to the changed underlying address.
+ */
+ auto operator+=(difference_type value) -> memory_area_iterator &;
+
+ /**
+ * @brief Subtraction assignment operator. Returns a reference to the changed value.
+ *
+ * @param value Value we want to subtract from the underlying address.
+ * @return Reference to the changed underlying address.
+ */
+ auto operator-=(difference_type value) -> memory_area_iterator &;
+
+ /**
+ * @brief Addition operator. Returns the changed value.
+ *
+ * @param value Value we want to add to a copy of the underlying address.
+ * @return Copy of underlying address incremented by the given value.
+ */
+ auto operator+(difference_type value) const -> memory_area_iterator;
+
+ /**
+ * @brief Subtraction operator. Returns the changed value.
+ *
+ * @param value Value we want to subtrcat from a copy of the underlying address.
+ * @return Copy of underlying address decremented by the given value.
+ */
+ auto operator-(difference_type value) const -> memory_area_iterator;
+
+ /**
+ * @brief Subtraction operator. Returns the size difference between two iterators.
+ *
+ * @param other Other iterator we want to substract the underlying address with ours.
+ * @return Size difference between the underlying address of this instance and the given iterator.
+ */
+ auto operator-(const memory_area_iterator & other) const -> difference_type;
+
+ /**
+ * @brief Index operator overload. Returns a reference to the value at the given index. Simply returns the
+ * dereferenced underlying pointer incremented by the given index.
+ *
+ * @param index Index we want to access and get the value from.
+ * @return Reference to the value at the given index.
+ */
+ auto operator[](difference_type index) const -> value_type &;
/**
* @brief Defaulted comparsion operator. Simply compares the memory address of both iterators.
*
* @param other Other iterator to compare to.
- * @return Whether poith iterators point to the same underlying address in memory.
+ * @return Whether both iterators point to the same underlying address in memory.
+ */
+ auto operator==(memory_area_iterator const & other) const -> bool = default;
+
+ /**
+ * @brief Defaulted threeway comparsion operator. Simply compares the memory address of both iterators.
+ *
+ * @param other Other iterator to compare to.
+ * @return Whether the given iterator is smaller or larger than this iterator.
+ */
+ auto operator<=>(memory_area_iterator const & other) const -> std::strong_ordering = default;
+
+ private:
+ value_type * ptr; ///< Underlying address the iterator is currently pointing too.
+ };
+
+ /**
+ * @brief Read-only container for memory areas, that allow to easily use the memory_area_iterator in C++20 ranges
+ * calls.
+ */
+ struct memory_area_container
+ {
+ using iterator = memory_area_iterator; ///< Iterators used by this container.
+ using size_type = std::size_t; ///< Maximum size of this container.
+
+ /**
+ * @brief Constructor.
+ *
+ * @param begin Pointer to the first memory area, will be used to construct the begin iterator.
+ * @param size Amount of entries in the container we want to construct.
+ */
+ memory_area_container(memory_area_iterator::value_type * begin, size_type size);
+
+ /**
+ * @brief Returns the iterator pointing to the first element of the memory area.
+ * Allows using this class in the for each loop, because it follows the InputIterator template scheme.
+ *
+ * @return Iterator pointing to first element of the memory area.
+ */
+ auto begin() const -> iterator;
+
+ /**
+ * @brief Returns the iterator pointing to one past the last element of the memory area.
+ * Allows using this class in the for each loop, because it follows the InputIterator template scheme.
+ *
+ * @return Iterator pointing to one past the last element of the memory area.
+ */
+ auto end() const -> iterator;
+
+ /**
+ * @brief Calculates the size of this container, simply subtracts the iterator pointing to the first element by the
+ * last.
+ *
+ * @return Actual size of this container.
+ */
+ auto size() const -> size_type;
+
+ /**
+ * @brief Calcualtes the size and returns true if the size is 0 and the container therefore emtpy.
+ *
+ * @return Whether the container is empty, size being 0 or not
*/
- bool operator==(memory_area_iterator const & other) const = default;
+ auto empty() const -> bool;
private:
- memory_area * ptr; ///< Underlying address the iterator is currently pointing too.
+ iterator area_begin; ///< Pointer to the first element of all memory areas.
+ iterator area_end; ///< Pointer to one pas the last element of all memory areas.
};
} // namespace teachos::arch::memory::multiboot
@@ -44,48 +45,175 @@ namespace teachos::arch::memory::multiboot
};
/**
- * @brief Iterator for memory areas.
+ * @brief Random access iterator for memory areas.
*/
struct memory_area_iterator